<?php

namespace Drupal\Tests\google_calendar_service\Functional;

use Drupal\google_calendar_service\Entity\CalendarEvent;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the Google Calendar Service Admin Pages.
 *
 * @group google_calendar_service
 */
class GoogleCalendarServiceUserPagesTest extends BrowserTestBase {

  /**
   * Modules to enable.
   *
   * @var array<string>
   */
  protected static $modules = ['google_calendar_service', 'node'];

  /**
   * Theme to enable. This field is mandatory.
   *
   * @var string
   */
  protected $defaultTheme = 'stark';

  /**
   * A test user with permission to access google calendar service config page.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * A test user with permission to see the /calendar/events page.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $regularUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    // Make sure to complete the normal setup steps first.
    parent::setUp();

    // Create and log in as an administrative user.
    $this->adminUser = $this->drupalCreateUser([
      'administer calendars and events',
      'add calendars',
      'add events',
      'delete calendars',
      'delete events',
      'edit calendars',
      'edit events',
      'view calendars',
      'view events',
    ]);

    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests if user can load the front page among other things.
   */
  public function testLoadSite() {
    // Load the front page.
    $this->drupalGet('<front>');

    // Confirm that the site didn't throw a server error or something else.
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet("user");
  }

  /**
   * Tests the /admin/calendar page.
   */
  public function testAdminCalendarPage() {
    $this->drupalGet('admin/calendar');

    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->pageTextContains('Name');
    $session->pageTextContains('Google Calendar ID');
    $session->pageTextContains('Operations');

    /* Add a Calendar Entity */
    $this->drupalGet("admin/calendar/add");
    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->buttonExists("Save");

    $edit = [
      'name[0][value]' => 'Test1',
      'calendar_id[0][value]' => 'googletest@gmail.com',
    ];

    $this->submitForm($edit, 'Save');
    $session->pageTextContains('Created the');
    $session->pageTextContains("Test1");

    /* Add Another Entity */
    $this->drupalGet("admin/calendar/add");
    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->buttonExists("Save");

    $edit = [
      'name[0][value]' => 'Test2',
      'calendar_id[0][value]' => 'googletest2@gmail.com',
    ];

    $this->submitForm($edit, 'Save');
    $session->pageTextContains('Created the');
    $session->pageTextContains("Test2");
  }

  /**
   * Test the /calendar/1/events page.
   */
  public function testCalendarEventsPage() {
    /* Let's create the event first */
    $this->drupalGet('admin/calendar');

    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->pageTextContains('Name');
    $session->pageTextContains('Google Calendar ID');
    $session->pageTextContains('Operations');

    /* Add a Calendar Entity */
    $this->drupalGet("admin/calendar/add");
    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->buttonExists("Save");

    $edit = [
      'name[0][value]' => 'Test1',
      'calendar_id[0][value]' => 'googletest@gmail.com',
    ];

    $this->submitForm($edit, 'Save');
    $session->pageTextContains('Created the');
    $session->pageTextContains("Test1");

    /* Let's create a few events for the Calendar 1 */
    $event_1 = CalendarEvent::create([
      'name'  => 'Event 1 Calendar 1',
      'event_id' => 1,
      'calendar' => 1,
      'description' => "This is the event description for Event 1",
      'start_date' => time() + (60 * 60),
      'end_date' => time() + (60 * 60 * 24),
      'event_url' => '',
    ])->save();

    $event_2 = CalendarEvent::create([
      'name'  => 'Event 2 Calendar 1',
      'event_id' => 1,
      'calendar' => 1,
      'description' => "This is the event description for Event 2",
      'start_date' => time() + (60 * 60),
      'end_date' => time() + (60 * 60 * 24),
      'event_url' => '',
    ])->save();

    $event_3 = CalendarEvent::create([
      'name'  => 'Event 3 Calendar 1',
      'event_id' => 1,
      'calendar' => 1,
      'description' => "This is the event description for Event 3",
      'start_date' => time() + (60 * 60),
      'end_date' => time() + (60 * 60 * 24),
      'event_url' => '',
    ])->save();

    $this->drupalGet('calendar/1/events');

    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->pageTextContains('Event Name');
    $session->pageTextContains('Start Date');
    $session->pageTextContains('End Date');
    $session->pageTextContains('Event 1 Calendar 1');
    $session->pageTextContains('Event 2 Calendar 1');

    /* Logout to see the pages as regular user */

    $this->drupalLogout($this->adminUser);

    $this->regularUser = $this->drupalCreateUser([
      'view calendars',
      'view events',
    ]);

    $this->drupalLogin($this->regularUser);

    /* Load a specific calendar */
    $this->drupalGet('calendar/1/events');

    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->pageTextContains('Event 1 Calendar 1');
    $session->pageTextContains('Event 2 Calendar 1');
    $session->pageTextContains('Event 3 Calendar 1');

    /* Load the views calendar */
    $this->drupalGet('calendar/events');

    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->pageTextContains('Event 1 Calendar 1');
    $session->pageTextContains('Event 2 Calendar 1');
    $session->pageTextContains('Event 3 Calendar 1');

    $edit = [
      'search' => 'Event 2 Calendar 1',
    ];

    $this->submitForm($edit, 'Apply');
    $session->pageTextContains('Event 2 Calendar 1');

  }

}
